# encoding: utf-8

# GNU GPL LICENSE
#
# This module is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; latest version thereof,
# available at: <http://www.gnu.org/licenses/gpl.txt>.
#
# This module is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this module; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA

"""
XML-RPC library for the test client.

Created by Florian Leitner on 2009-05-10 at CNIO.
Copyright (c) 2009 Florian Leitner. All rights reserved.
"""

import logging
import xmlrpclib

__author__ = "Florian Leitner"
__version__ = "1.0"

class ServerClient(object):
    """
    Extended xmlrpclib.ServerProxy to send Fulltext or Medline
    requests using a timeout value on the connection socket.
    """
    def __init__(self, uri, timeout):
        super(ServerClient, self).__init__()
        self.uri = uri
        self.l = logging.getLogger("Client")
        self.proxy = xmlrpclib.ServerProxy(
            uri, transport=TimeoutTransport(timeout=timeout)
        )
        self.l.debug("using URL %s with a timeout of %i seconds" % (
            uri, timeout
        ))
    
    def fulltext_request(self, text_blocks):
        self.l.debug("starting a Fulltext request")
        return self.proxy.Fulltext.getAnnotation(text_blocks)
    
    def medline_request(self, pubmed_id):
        self.l.debug("starting a Medline request")
        return self.proxy.Medline.getAnnotation(pubmed_id)
    

class TimeoutTransport(xmlrpclib.Transport):
    """Sets the timeout attibute on any HTTP(S) connections through HTTP."""
    
    def __init__(self, use_datetime=0, timeout=60.0):
        xmlrpclib.Transport.__init__(self, use_datetime)
        self.timeout = timeout
    
    def make_connection(self, host):
        """
        Extends Transport.make_connection to get a timeout on the
        socket by setting the attribute on the connection instance.
        """
        http = xmlrpclib.Transport.make_connection(self, host)
        http._conn.timeout = self.timeout
        return http
    

