package biocreative.webservice;

import java.io.IOException;

import java.net.URL;

import org.apache.xmlrpc.XmlRpcException;
import org.apache.xmlrpc.server.PropertyHandlerMapping;
import org.apache.xmlrpc.server.XmlRpcServer;
import org.apache.xmlrpc.server.XmlRpcServerConfigImpl;
import org.apache.xmlrpc.webserver.WebServer;

/**
 * Standalone AnnotationServer implementaion using the Apache XML-RPC
 * WebServer class.
 * 
 * This is the secondary server to use. To start a new server process, use
 * <code>java biocreative.webservice.AnnotationServer</code> and the server
 * runs 'forever'.
 *
 * As opposed to the Servlet, this allows you to define your own
 * location of the Apache XML-RPC Handler propery file
 * XmlRpcServlet.properties, which should be in org.apache.xmlrpc.webserver
 * by default.
 */
public class AnnotationServer {

    public static void main(String[] args) {

        /* Initialize the AnnotationServer.properties */
        ServerProperties serverProps = null;
        try {
            serverProps = new ServerProperties();
            /* alternatively, you can specify another location than default
             * by using this line:
             * ServerProperties serverProps =
             *     new ServerProperties("my/path/to/server.properties");
             */
        } catch (Exception ex) {
            ex.printStackTrace(System.err);
            System.exit(1);
        }

        /* Initialize the Server */
        WebServer server = new WebServer(
            serverProps.getPort(), serverProps.getServerIP()
        );
        
        /* Optional settings to enhance security */
        /*
        server.setParanoid(true);
        if (serverProps.getClientIPs() != null) {
            for (Object ip: serverProps.getClientIPs()) {
                server.acceptClient((String) ip);
            }
        }
        */
        
        XmlRpcServer xmlRpcServer = server.getXmlRpcServer();
        XmlRpcServerConfigImpl serverConfig =
                (XmlRpcServerConfigImpl) xmlRpcServer.getConfig();
        serverConfig.setEnabledForExtensions(true);
        serverConfig.setContentLengthOptional(false);

        /* Initialize the property handler file */
        PropertyHandlerMapping phm = new PropertyHandlerMapping();
        String handlerPropsPath = serverProps.getHandlerProperties();
        if (handlerPropsPath == null) {
            handlerPropsPath =
                    "org/apache/xmlrpc/webserver/XmlRpcServlet.properties";
        }
        try {
            phm.load(Thread.currentThread().getContextClassLoader(),
                     serverProps.getHandlerProperties());
        } catch (IOException ex) {
            ex.printStackTrace(System.err);
            System.exit(1);
        } catch (XmlRpcException ex) {
            ex.printStackTrace(System.err);
            System.exit(1);
        }
        xmlRpcServer.setHandlerMapping(phm);

        /* Start the web service */
        try {
            server.start();
        } catch (IOException ex) {
            ex.printStackTrace(System.err);
            System.exit(1);
        }
    }
}