package biocreative.webservice;

import java.io.InputStream;
import java.io.IOException;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.net.URL;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

/**Parser for the AnnotationServer.properties file. */
public class ServerProperties extends Properties {

    /**
     * generated serial version UID
     */
    private static final long serialVersionUID = 0L;

    /** Default location of the server property file. */
    public static final String defaultPropertyPath =
            "biocreative/webservice/AnnotationServer.properties";

    private InetAddress serverIP = null;

    private List clientIPs = null;

    private int port = -1;

    private String handlerProperties = null;

    /** Instantiates the server properties using the default file location. */
    public ServerProperties() throws UnknownHostException,
            NumberFormatException, RuntimeException, IOException {
        this(defaultPropertyPath);
    }

    /** Instantiates the server properties using a different file location. */
    public ServerProperties(String propertyPath) throws UnknownHostException,
            NumberFormatException, RuntimeException, IOException {
        super();
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        URL resource = cl.getResource(propertyPath);
        if (resource == null)
            throw new RuntimeException("not a valid URL resource: '"
                + propertyPath + "'");
        initialize(resource.openStream());
    }

    protected void initialize(InputStream stream) throws UnknownHostException,
            NumberFormatException, RuntimeException, IOException {
        super.load(stream);
        for (Map.Entry entry : this.entrySet()) {
            String key = (String) entry.getKey();
            String value = (String) entry.getValue();
            if (key.equals("serverIP")) {
                this.setServerIP(value);
            } else if (key.equals("serverPort")) {
                this.setPort(value);
            } else if (key.equals("clientIPs")) {
                this.setClientIPs(value);
            } else if (key.equals("handlerProperties")) {
                this.setHandlerProperties(value);
            } else {
                throw new RuntimeException("no such server property: '" + key
                    + "'");
            }
        }
    }

    public void setServerIP(String serverIP) throws UnknownHostException {
        this.serverIP = InetAddress.getByName(serverIP);
    }

    public InetAddress getServerIP() {
        return this.serverIP;
    }

    public void setClientIPs(String clientIPs) throws UnknownHostException {
        // test IPs are valid
        if (this.clientIPs == null)
            this.clientIPs = new ArrayList();
        for (String ip : clientIPs.trim().split("\\s*,\\s*")) {
                if (! ip.matches(".*?\\*$"))
                    InetAddress.getByName(ip); // test IPs is valid
                this.clientIPs.add(ip);
            }
    }

    public List getClientIPs() {
        return this.clientIPs;
    }

    public void setPort(String port) throws NumberFormatException {
        this.port = Integer.parseInt(port);
    }

    public int getPort() {
        return this.port;
    }

    /**
     * Handler property file location is useless if you are using the Servlet
     * server! For a Servlet the properties are always loaded from
     * org/apache/xmlrpc/webserver/XmlRpcServlet.properties.
     */
    public void setHandlerProperties(String propertyPath) {
        this.handlerProperties = propertyPath;
    }

    public String getHandlerProperties() {
        return this.handlerProperties;
    }
}